/**************************************************************************//**
 * @file     main.c
 * @version  V1.00
 * @brief    M051 Simulate UART transfers using GPIO Sample Code
 *
 * @note
 * Copyright (C) 2019 Nuvoton Technology Corp. All rights reserved.
 ******************************************************************************/
#include <stdio.h>
#include "M051Series.h"
#include "SoftwareUART.h"

#define PLLCON_SETTING      CLK_PLLCON_50MHz_HXT
#define PLL_CLOCK           50000000


void SYS_Init(void)
{
    /*---------------------------------------------------------------------------------------------------------*/
    /* Init System Clock                                                                                       */
    /*---------------------------------------------------------------------------------------------------------*/
    
	  /* Enable IRC22M clock */
    CLK_EnableXtalRC(CLK_PWRCON_OSC22M_EN_Msk);
	
    /* Waiting for IRC22M clock ready */
    CLK_WaitClockReady(CLK_CLKSTATUS_IRC22M_STB_Msk);

    /* Enable external 12MHz XTAL clock ready */
    CLK_EnableXtalRC(CLK_PWRCON_XTL12M_EN_Msk);

    /* Waiting for 12MHz XTAL clock ready */
    CLK_WaitClockReady(CLK_CLKSTATUS_XTL12M_STB_Msk);

    /* Enable external IRC10K */
    CLK_EnableXtalRC(CLK_PWRCON_OSC10K_EN_Msk);
    
    /* Waiting for IRC10K clock ready */
    CLK_WaitClockReady(CLK_CLKSTATUS_OSC10K_STB_Msk); 
		
    /* Set core clock as PLL_CLOCK from PLL */
    CLK_SetCoreClock(PLL_CLOCK);

    /* Switch HCLK clock source to HIRC */
    CLK_SetHCLK(CLK_CLKSEL0_HCLK_S_PLL, CLK_CLKDIV_HCLK(1));

    /* Enable peripheral clock */
    /* Enable UART0 module clock */ 
    CLK_EnableModuleClock(UART0_MODULE);
    /* Enable TIMER0 module clock */ 
    CLK_EnableModuleClock(TMR0_MODULE);
    /* Enable TIMER1 module clock */
    CLK_EnableModuleClock(TMR1_MODULE); 
		
    /* Peripheral clock source */
	
    /* Select UART module clock source */
    CLK_SetModuleClock(UART0_MODULE, CLK_CLKSEL1_UART_S_HXT, CLK_CLKDIV_UART(1));
    /* Select TMR0 module clock source */
    CLK_SetModuleClock(TMR0_MODULE, CLK_CLKSEL1_TMR0_S_HXT,0);
    /* Select TMR1 module clock source */
    CLK_SetModuleClock(TMR1_MODULE, CLK_CLKSEL1_TMR1_S_HXT,0);

    /* Update System Core Clock */
    /* User can use SystemCoreClockUpdate() to calculate PllClock, SystemCoreClock and CycylesPerUs automatically. */
    SystemCoreClockUpdate();

    /*---------------------------------------------------------------------------------------------------------*/
    /* Init I/O Multi-function                                                                                 */
    /*---------------------------------------------------------------------------------------------------------*/
    /* Set P3 multi-function pins for UART0 RXD, TXD */
    SYS->P3_MFP = SYS_MFP_P30_RXD0 | SYS_MFP_P31_TXD0;
}

void SoftwareUART_TestItem(void)
{

    printf("+--------------------------------------------------------+\n");
    printf("|           M051 SoftwareUART Transmission               |\n");
    printf("+--------------------------------------------------------+\n");
    printf("| [1] Software UART TX                                   |\n");
    printf("| [2] Software UART RX                                   |\n");
    printf("+--------------------------------------------------------+\n");
    printf("Please Enter any Key to Start: "); 
}	
/*---------------------------------------------------------------------------------------------------------*/
/* UART0 Init Function                                                                                     */
/*---------------------------------------------------------------------------------------------------------*/
void UART0_Init(void)
{
    /* Reset IP */
    SYS_ResetModule(UART0_RST);

    /* Configure UART0 and set UART0 Baudrate */
    UART_Open(UART0, 115200);
}
/*---------------------------------------------------------------------------------------------------------*/
/* Simulate UART Tx transfers using GPIO                                                                   */
/*---------------------------------------------------------------------------------------------------------*/
void UART_TX_TestFunctoion(sUART_Config *psConfig)
{
   printf("Transfer %d-byte serial number\n",psConfig->u32SendLen);
   SoftwareUART_SendData(psConfig);
}	

/*---------------------------------------------------------------------------------------------------------*/
/*  Receive data using GPIO emulation UART Rx                                                              */
/*---------------------------------------------------------------------------------------------------------*/
void UART_RX_TestFunctoion(sUART_Config *psConfig)
{	
   printf("Waiting to receive %d-byte\n",psConfig->u32ReceiveLen);
   SoftwareUART_ReadData(psConfig);
}	


/*---------------------------------------------------------------------------------------------------------*/
/*  MAIN function                                                                                          */
/*---------------------------------------------------------------------------------------------------------*/
int main(void)
{
    uint8_t u8TestItem;
    /* Unlock protected registers */
    SYS_UnlockReg();

    /* Init System, peripheral clock and multi-function I/O */
    SYS_Init();

    /* Lock protected registers */
    SYS_LockReg();
    
    /* Init UART0 for printf */
    UART0_Init();
	  /* Init SoftwareUART  */
    SoftwareUART_Init(&g_sSoftwareUART_Config);
    printf("\nSystem Clock = %d Hz\n",SystemCoreClock);
    
   while(1)
   {

    SoftwareUART_TestItem();
    u8TestItem = getchar(); 
    printf("%c\n",u8TestItem);
    
     switch(u8TestItem)
     {
       case '1':
               UART_TX_TestFunctoion(g_psSoftwareUART_Config);
              break; 
       case '2':
               UART_RX_TestFunctoion(g_psSoftwareUART_Config);
              break;
       default:
              break; 				 
     } 
    }
}

/*** (C) COPYRIGHT 2019 Nuvoton Technology Corp. ***/

